﻿Public Class Form1
    Inherits Form

    Dim player As New Player()
    Dim dealer As New Dealer()
    Dim deck As New List(Of String)
    Dim gameRunning As Boolean = True
    Private Sub StartGame_Click(sender As Object, e As EventArgs) Handles StartGame.Click
        PlayGame()
    End Sub
    Private Sub HitButton_Click(sender As Object, e As EventArgs) Handles HitButton.Click
        If gameRunning And player.hand.Count < 5 And player.ingame Then
            DealCard(player)
            CheckPlayer(player)
        End If
    End Sub

    Private Sub StandButton_Click(sender As Object, e As EventArgs) Handles StandButton.Click
        If gameRunning And player.ingame Then
            player.ingame = False
            DealerTurn()
        End If
    End Sub
    Private Sub ResetCards()
        Dim pictureBoxes() As PictureBox = {PlayerCard1, PlayerCard2, PlayerCard3, PlayerCard4, PlayerCard5, PlayerCard6, PlayerCard7, PlayerCard8, PlayerCard9, PlayerCard10}
        For Each box In pictureBoxes
            box.Image = Nothing
        Next
    End Sub
    Sub PlayGame()
        InitGame()
    End Sub
    Sub InitGame()
        ResetCards()
        WinMessage.ResetText()
        HitButton.Enabled = True
        StandButton.Enabled = True
        StartGame.Enabled = False
        gameRunning = True
        player.Start()
        dealer.Start()
        CreateDeck()
        DealCard(player)
        DealCard(dealer, isDealer:=True)
        DealCard(player)
        DealCard(dealer, isDealer:=True)
        CheckPlayer(player)
    End Sub
    Sub CreateDeck()
        deck.Clear()
        Dim nums As String() = {"A", "2", "3", "4", "5", "6", "7", "8", "9", "10", "J", "Q", "K"}
        Dim suits As String() = {"Hearts", "Diamonds", "Spades", "Clubs"}
        For Each s As String In suits
            For Each n As String In nums
                deck.Add(n + " of " + s)
            Next
        Next
        Shuffle(deck)
    End Sub
    Sub Shuffle(deck)
        Dim max As Integer = deck.Count - 1
        Dim random As New Random()
        For x As Integer = 0 To max
            Dim rand As Integer = random.Next(0, max)
            Dim temp As String = deck(x)
            deck(x) = deck(rand)
            deck(rand) = temp
        Next
    End Sub
    Sub DealCard(player, Optional suppress = False, Optional isDealer = False)
        Dim card = deck(0)
        player.hand.Add(card)
        deck.RemoveAt(0)
        GetTotal(player)
        Dim cardNumber As String = If(isDealer, (player.hand.count + 5).ToString(), player.hand.count)
        Dim pictureBox As PictureBox
        Try
            pictureBox = CType(Me.Controls.Find("PlayerCard" + cardNumber, True).First(), PictureBox)
        Catch ex As Exception
            WinMessage.Text = "Error: " + ex.Message
            EndGame()
        End Try

        pictureBox.Image = GetCardImage(card)
        SetTotalLabels()
    End Sub
    Sub GetTotal(player)
        Dim cards As List(Of String) = player.hand
        If cards.Count = 0 Then
            player.total = 0
            Return
        End If
        Dim t As Integer = 0
        Dim a As Integer = 0
        For Each x As String In cards
            Dim card As String = x
            Dim num As String = card.Split(" "c)(0)
            If num = "K" Or num = "Q" Or num = "J" Then
                num = 10
            End If
            If num = "A" Then
                num = 11
                a += 1
            End If
            num = Integer.Parse(num)
            t += num
        Next
        While (a > 0 And t > 21)
            a -= 1
            t = t - 10
        End While
        player.total = t
    End Sub
    Sub SetTotalLabels()
        Dim dealerLabelTotal As String = DealerTotal.Text.Split(": ")(0)
        Dim playerLabelTotal As String = PlayerTotal.Text.Split(": ")(0)
        DealerTotal.Text = dealerLabelTotal + ": " + dealer.total.ToString()
        PlayerTotal.Text = playerLabelTotal + ": " + player.total.ToString()

    End Sub
    Sub CheckPlayer(player)
        If player.total = 21 And player.hand.count = 2 Then
            player.winType = WinCondition.Blackjack
            player.ingame = False
            dealer.ingame = False
            DealerTurn()
        ElseIf player.total > 21 Then
            player.winType = WinCondition.Bust
            player.ingame = False
            DealerTurn()
        ElseIf player.total < 22 And player.hand.count = 5 Then
            player.winType = WinCondition.FiveCard
            player.ingame = False
            dealer.ingame = False
            DealerTurn()
        End If
        If player.total = 21 Then
            player.ingame = False
            DealerTurn()
        End If
    End Sub
    Sub DealerTurn()
        If dealer.ingame Then
            If dealer.total = 21 And dealer.hand.Count = 2 Then
                dealer.winType = WinCondition.Blackjack
                dealer.ingame = False
            ElseIf dealer.total > 21 Then
                dealer.winType = WinCondition.Bust
                player.ingame = False
            ElseIf dealer.total < dealer.limit Then
                DealCard(dealer, isDealer:=True)
                DealerTurn()
            Else
                dealer.winType = WinCondition.NormalWin
            End If
        End If
        CheckGame()
    End Sub
    Sub CheckGame()
        WinMessage.Text = GetResults()
        EndGame()
    End Sub
    Sub EndGame()
        HitButton.Enabled = False
        StandButton.Enabled = False
        StartGame.Enabled = True
    End Sub
    Function GetResults()
        If dealer.winType = WinCondition.Blackjack Then
            If player.winType = WinCondition.Blackjack Then
                PrintCurrentGame()
                Return "Player ties with Blackjack!"
            Else
                PrintCurrentGame()
                Return "Dealer wins with Blackjack!"
            End If
        End If
        If player.winType = WinCondition.Blackjack Then
            PrintCurrentGame()
            Return "Player wins with Blackjack!"
        End If
        If player.winType = WinCondition.FiveCard Then
            PrintCurrentGame()
            Return "Player wins with Five Card Charlie!"
        End If
        If player.winType = WinCondition.Bust Then
            PrintCurrentGame()
            Return "Player busts!"
        End If
        If dealer.winType = WinCondition.Bust Then
            PrintCurrentGame()
            Return "Dealer busts!"
        End If
        If player.total > dealer.total Then
            PrintCurrentGame()
            Return "Player wins with total of " + player.total.ToString() + "!"
        End If
        If player.total = dealer.total Then
            PrintCurrentGame()
            Return "Tie!"
        End If
        If player.total < dealer.total Then
            PrintCurrentGame()
            Return "Dealer wins with total of " + dealer.total.ToString() + "!"
        End If
        PrintCurrentGame()
        Return "error"
    End Function
    Function GetCardImage(card) As Image
        Dim num As String = card.split(" ")(0)
        Dim suit As String = card.split(" ")(2)
        Select Case num
            Case "K"
                num = "king"
            Case "Q"
                num = "queen"
            Case "J"
                num = "jack"
            Case "A"
                num = "ace"
        End Select

        Dim img As String = suit + "_" + num
        Return My.Resources.ResourceManager.GetObject(img.ToLower())
    End Function
    Sub PrintCurrentGame()
        Console.WriteLine("Dealer Hand: " + String.Join(", ", dealer.hand) + " Total: " + dealer.total.ToString())
        Console.WriteLine("Player Hand: " + String.Join(", ", player.hand) + " Total: " + player.total.ToString())
    End Sub


End Class

Public Class Player
    Private _winCondition As WinCondition
    Public Property hand As New List(Of String)
    Public Property total As Integer
    Public Property ingame As Boolean
    Public Property winType
        Get
            Return _winCondition
        End Get
        Set
            _winCondition = Value
        End Set
    End Property

    Sub Start()
        hand.Clear()
        total = 0
        ingame = True
        winType = WinCondition.NormalWin

    End Sub
End Class
Public Class Dealer
    Inherits Player

    Public Property limit As Integer = 17
End Class

Enum WinCondition
    Bust
    NormalWin
    FiveCard
    Blackjack
End Enum